#!/bin/sh -xe

mkdir -p /usr/local/src
cd /usr/local/src

pkg_add wget
wget https://github.com/concurrencykit/ck/archive/refs/tags/0.7.2.tar.gz
tar zxf 0.7.2.tar.gz
cd ck-0.7.2
./configure
make
make install
